@@ -11,9 +11,9 @@ module Network.Wai.Handler.Warp.File (
 
 import Control.Applicative ((<|>))
 import Data.Array ((!))
-import Data.ByteString (ByteString)
 import qualified Data.ByteString as B hiding (pack)
 import qualified Data.ByteString.Char8 as B (pack, readInteger)
+import Data.ByteString (ByteString)
 import Data.Maybe (fromMaybe)
 import Network.HTTP.Date
 import qualified Network.HTTP.Types as H
@@ -21,12 +21,16 @@ import qualified Network.HTTP.Types.Header as H
 import Network.Wai
 import qualified Network.Wai.Handler.Warp.FileInfoCache as I
 import Network.Wai.Handler.Warp.Header
+import Network.Wai.Handler.Warp.PackInt
 import Numeric (showInt)
 
 #ifndef MIN_VERSION_http_types
 #define MIN_VERSION_http_types(x,y,z) 1
 #endif
 
+-- $setup
+-- >>> import Test.QuickCheck
+
 ----------------------------------------------------------------
 
 data RspFileInfo = WithoutBody H.Status
@@ -40,17 +44,17 @@ conditionalRequest :: I.FileInfo
                    -> RspFileInfo
 conditionalRequest finfo hs0 reqidx = case condition of
     nobody@(WithoutBody _) -> nobody
-    WithBody s _ off len   -> let hs = (H.hLastModified,date) :
-                                       addContentHeaders hs0 off len size
+    WithBody s _ off len   -> let !hs = (H.hLastModified,date) :
+                                        addContentHeaders hs0 off len size
                               in WithBody s hs off len
   where
-    mtime = I.fileInfoTime finfo
-    size  = I.fileInfoSize finfo
-    date  = I.fileInfoDate finfo
-    mcondition = ifmodified    reqidx size mtime
-             <|> ifunmodified  reqidx size mtime
-             <|> ifrange       reqidx size mtime
-    condition = fromMaybe (unconditional reqidx size) mcondition
+    !mtime = I.fileInfoTime finfo
+    !size  = I.fileInfoSize finfo
+    !date  = I.fileInfoDate finfo
+    !mcondition = ifmodified    reqidx size mtime
+              <|> ifunmodified  reqidx size mtime
+              <|> ifrange       reqidx size mtime
+    !condition = fromMaybe (unconditional reqidx size) mcondition
 
 ----------------------------------------------------------------
 
@@ -70,7 +74,7 @@ ifmodified reqidx size mtime = do
     date <- ifModifiedSince reqidx
     return $ if date /= mtime
              then unconditional reqidx size
-             else  WithoutBody H.notModified304
+             else WithoutBody H.notModified304
 
 ifunmodified :: IndexedHeader -> Integer -> HTTPDate -> Maybe RspFileInfo
 ifunmodified reqidx size mtime = do
@@ -140,16 +144,17 @@ parseByteRanges bs1 = do
 
 ----------------------------------------------------------------
 
+contentRange :: H.HeaderName
+#if MIN_VERSION_http_types(0,9,0)
+contentRange = H.hContentRange
+#else
+contentRange = "Content-Range"
+#endif
+
 -- | @contentRangeHeader beg end total@ constructs a Content-Range 'H.Header'
 -- for the range specified.
 contentRangeHeader :: Integer -> Integer -> Integer -> H.Header
-contentRangeHeader beg end total = (
-#if MIN_VERSION_http_types(0,9,0)
-        H.hContentRange
-#else
-        "Content-Range"
-#endif
-    , range)
+contentRangeHeader beg end total = (contentRange, range)
   where
     range = B.pack
       -- building with ShowS
@@ -161,19 +166,21 @@ contentRangeHeader beg end total = (
       ( '/'
       : showInt total "")
 
-addContentHeaders :: H.ResponseHeaders -> Integer -> Integer -> Integer -> H.ResponseHeaders
-addContentHeaders hs off len size = hs''
-  where
-    contentRange = contentRangeHeader off (off + len - 1) size
-    !lengthBS = B.pack $ show len
-    hs' = (H.hContentLength, lengthBS):(
+acceptRange :: H.HeaderName
 #if MIN_VERSION_http_types(0,9,0)
-        H.hAcceptRanges
+acceptRange = H.hAcceptRanges
 #else
-        "Accept-Ranges"
+acceptRange = "Accept-Ranges"
 #endif
-        , "bytes"):hs
-    hs'' = if len == size then hs' else contentRange:hs'
+
+addContentHeaders :: H.ResponseHeaders -> Integer -> Integer -> Integer -> H.ResponseHeaders
+addContentHeaders hs off len size
+  | len == size = hs'
+  | otherwise   = let !ctrng = contentRangeHeader off (off + len - 1) size
+                  in ctrng:hs'
+  where
+    !lengthBS = packIntegral len
+    !hs' = (H.hContentLength, lengthBS) : (acceptRange,"bytes") : hs
 
 -- |
 --
